#define STRICT
#define ORBITER_MODULE
#define DOCKSTRUCTOFFSET_CURRENTDOCKOBJECT 0x48
#define ADDRESS_GETDOCKSTATUS 0x004217E0

#include <stdio.h>
#include <map>
#include "orbitersdk.h"
#include "UMMUFB.h"
#include "PAX.h"

// ==============================================================
// Global variables

int g_MFDmode; // identifier for new MFD mode
DWORD g_Hook;
OBJHANDLE g_Transfer;
byte g_original[10]={0x8b,0x44,0x24,0x04,0x8b,0x40,0x48,0xc2,0x04,0x00};
//The following array is:
//_asm
//{
//	pop eax;
//	push ecx;
//	push eax;
//	jmp dword ptr [GetDockStatus]; //Dynamically detected address
//	nop;
//}
byte g_code[10] = {0x58, 0x51, 0x50, 0xff, 0x25, 0, 0, 0, 0, 0x90};

typedef std::map<VESSEL*, OBJHANDLE> BaseByVesselType;
BaseByVesselType& BaseByVessel()
{
	static BaseByVesselType g_BaseByVessel;
	return g_BaseByVessel;
};

void GetLandedPad(VESSEL *vessel, OBJHANDLE &base, DWORD &port)
{
	VESSELSTATUS2 s;
	s.version=2;
	s.flag=0;
	vessel->GetStatusEx(&s);
	base=NULL;
	port=0;
	if (s.status==1 && s.rbody!=NULL)
	{
		int k=oapiGetBaseCount(s.rbody);
		VECTOR3 distance;
		OBJHANDLE nearest=NULL;
		double nearestDistance=-1;
		for(int i=0;i<k;i++)
		{
			base=oapiGetBaseByIndex(s.rbody, i);
			int l=oapiGetBasePadCount(base);			
			vessel->GetRelativePos(base, distance);
			double len=length(distance);
			if (nearestDistance<0 || nearestDistance>len)
			{
				nearestDistance=len;
				nearest=base;
			}			
		}
		if (nearest!=NULL)
		{
			k=oapiGetBasePadCount(nearest);
			double vlng, vlat, vrad;
			vessel->GetEquPos(vlng, vlat, vrad);
			for(int i=0;i<k;i++)
			{
				double lng, lat;
				oapiGetBasePadEquPos(nearest, i, &lng, &lat);
				lng-=vlng;
				lat-=vlat;
				if (sqrt(lng*lng+lat*lat)*vrad<50)
				{
					port=i+1;
					break;
				}
			}
		}
		if (port>0) base=nearest;
		else base=NULL;
	}
}

OBJHANDLE _stdcall GetDockStatus(VESSEL *vessel, DOCKHANDLE dock)
{
	//Do my own GetDockStatus
	std::map<VESSEL *, OBJHANDLE>::iterator el=BaseByVessel().find(vessel);
	if (el!=BaseByVessel().end())
	{
		if (IsUMmuBase(vessel)) return el->second;
		OBJHANDLE base;
		DWORD port;
		GetLandedPad(vessel, base, port);
		if (port>0) return el->second;
		else DelPAX(vessel);		
	}
	//Original function content
	return *(OBJHANDLE *)(void *)((char *)dock+DOCKSTRUCTOFFSET_CURRENTDOCKOBJECT);
}

int WriteCode(void *address, void *code, DWORD len)
{
	//Get process information
	HANDLE hSelf = OpenProcess(PROCESS_ALL_ACCESS, FALSE, ::GetCurrentProcessId());
	MEMORY_BASIC_INFORMATION mbi;

	//Open up page of linked address
	if(VirtualQueryEx(hSelf, (LPVOID)address, &mbi, sizeof(mbi)) != sizeof(mbi)) return -1;
	PVOID pvRgnBaseAddress = mbi.BaseAddress;
	DWORD dwOldProtect1, dwOldProtect2, dwFake;
	if(!::VirtualProtectEx(hSelf, pvRgnBaseAddress, 4, PAGE_EXECUTE_READWRITE, &dwOldProtect1)) return -2;
	BOOL bStridePage = FALSE;
	LPBYTE lpByte = (LPBYTE)pvRgnBaseAddress;
	lpByte += 4096;
	if((DWORD)lpByte < (DWORD)address + 4) bStridePage = TRUE;
	PVOID pvRgnBaseAddress2 = (LPVOID)lpByte;
	if(bStridePage)
	   if(!::VirtualProtectEx(hSelf, pvRgnBaseAddress2, 4, PAGE_EXECUTE_READWRITE, &dwOldProtect2))
	   {
		   ::VirtualProtectEx(hSelf, pvRgnBaseAddress, 4, dwOldProtect1, &dwFake);
			return -3;
	   }
	
	//Write code
	memcpy(address, code, len);		

	//Lock vtables again
	::VirtualProtectEx(hSelf, pvRgnBaseAddress, 4, dwOldProtect1, &dwFake);
	if(bStridePage) ::VirtualProtectEx(hSelf, pvRgnBaseAddress2, 4, dwOldProtect2, &dwFake);
}

DLLCLBK void InitModule (HINSTANCE hDLL)
{
	//Hook
	union
	{
		void *pointer;
		byte bytes[4];
		DWORD value;
	} p;
	g_Hook=(DWORD)(void *)GetDockStatus;
	p.pointer=(void *)&g_Hook;	
	for(int i=0;i<4;i++) g_code[5+i] = p.bytes[i];
	if (memcmp((void *)g_original, (void *)ADDRESS_GETDOCKSTATUS, 10)==0)
		WriteCode((void *)ADDRESS_GETDOCKSTATUS, (void *)g_code, 10);	

	static char *name = "PAX";
	MFDMODESPEC spec;
	spec.name = name;
	spec.key = OAPI_KEY_P;
	spec.msgproc = PAX::MsgProc;

	// Register the new MFD mode with Orbiter
	g_MFDmode = oapiRegisterMFDMode (spec);
}

DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	// Unregister the custom MFD mode when the module is unloaded
	oapiUnregisterMFDMode (g_MFDmode);

	//Unhook
	if (memcmp((void *)g_code, (void *)ADDRESS_GETDOCKSTATUS, 10)==0)
		WriteCode((void *)ADDRESS_GETDOCKSTATUS, (void *)g_original, 10);
}

void SetPAX(VESSEL *vessel)
{
	OBJHANDLE me=vessel->GetHandle();
	for (int i=0;i<oapiGetVesselCount();i++)
	{
		OBJHANDLE base=oapiGetVesselByIndex(i);
		VESSEL *baseVessel=oapiGetVesselInterface(base);
		if (base!=me)
			if (IsUMmuBase(baseVessel))
		{
			BaseByVessel()[vessel]=base;
			BaseByVessel()[baseVessel]=vessel->GetHandle();
			break;
		}
	}
}

bool GetPAX(VESSEL *vessel)
{
	return BaseByVessel().find(vessel)!=BaseByVessel().end();
}

void DelPAX(VESSEL *vessel)
{
	std::map<VESSEL *, OBJHANDLE>::iterator el=BaseByVessel().find(vessel);
	if (el!=BaseByVessel().end())
	{
		BaseByVessel().erase(oapiGetVesselInterface(el->second));
		BaseByVessel().erase(el->first);
	}
}

bool IsUMmuBase(VESSEL *vessel)
{
	char *className=vessel->GetClassName();
	if (className!=NULL) if (strcmp(className, "UMmuBase")==0) return true;
	return false;
}