#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "UMmuSDK.h"
#include "UMmuBase.h"

void UMmuBase::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_hover, th_rcs[14], th_group[4];

	// physical specs
	SetSize (0.000001);
	SetEmptyMass (0.000001);
	SetCameraOffset (_V(0,0,0));
	SetDockParams (_V(0,1.3,-1), _V(0,1,0), _V(0,0,-1));
	SetTouchdownPoints (_V(0,0,1), _V(-1,0,-1), _V(1,0,-1));

	int UmmuSdKInitReturnCode=Crew.InitUmmu(GetHandle());	
	float UMmuVersion=Crew.GetUserUMmuVersion();
	Crew.DefineAirLockShape(TRUE,-2,2,0,4,-2,2);
	Crew.SetMembersPosRotOnEVA(_V(0,2,0),_V(0,0,0));
	Crew.SetMaxSeatAvailableInShip(100);
	SelectedUmmuMember=0;
	sprintf(display, "");
}

void UMmuBase::clbkSaveState(FILEHANDLE scn)
{
	SaveDefaultState (scn);
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}

void UMmuBase::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE) continue;
		ParseScenarioLineEx (line, status);
	}
}

void UMmuBase::clbkPostStep (double simtt, double simdt, double mjd)
{
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
		case UMMU_TRANSFERED_TO_OUR_SHIP: 
			sprintf(display,"%s \"%s\" aged %i was transfered to our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
		case UMMU_RETURNED_TO_OUR_SHIP:
			sprintf(display,"%s \"%s\" aged %i entered into our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
	}	
}

int UMmuBase::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if (!down) return 0; 

	if(key==OAPI_KEY_E)
	{
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));		
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(display,"Transfer to docked ship Ok - %s transfered",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(display,"EVA OK - %s left the ship",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(display,"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(display,"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(display,"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(display,"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(display,"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(display,"Error, docked ship do not use UMmu 1.5, ask author to add it See http://orbiter.dansteph.com");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(display,"Misc error with UMMU See http://orbiter.dansteph.com to install it again");
			break;
		}
		return TRUE;
	}

	if(key==OAPI_KEY_1)
	{
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(display,"Sorry no one aboard unable to select");	
			return 1;
		}

		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(display,"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	if(key==OAPI_KEY_2)
	{
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(display,"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(display,"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	if(key==OAPI_KEY_S)
	{
		sprintf(display,"%i souls aboard ship %s, %i seats available",Crew.GetCrewTotalNumber(),GetName(),100-Crew.GetCrewTotalNumber());
		return 1;
	}
	return 0;
}

void UMmuBase::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	TextOut(hDC, 0, 0, display, strlen(display));	
}

UMMUCREWMANAGMENT *UMmuBase::GetCrew()
{
	return &Crew;
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new UMmuBase (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (UMmuBase*)vessel;
}